#include <math.h>
#include <math_bf.h>
#include <fract_complex.h>
#include <fract2float_conv.h>
#include <stdlib.h>
#include <stdio.h>
#include "notes.h"
#include "g2mparameters.h"


static char note_names[25][4] = {"C4 ","C4#","D4 ","D4#","E4 ","F4 ","F4#","G4 ","G4#",
"A4 ","A4#","B4 ","C5 ","C5#","D5 ","D5#","E5 ", "F5 ","F5#","G5 ","G5#","A5 ","A5#","B5 ","C6 "};


pitch_t make_note(int time, pitch_t current_pitch, pitch_t last_pitch) {
	note_t note;

    if (last_pitch != SILENCE){
	    note.on = 0;
        note.time = time;
	    note.pitch = last_pitch;
	    fwrite(&note, sizeof(note_t), 1, stdout);
	    
	}
	
	if (current_pitch != SILENCE) {
		note.on = 1;
	    note.time = time + 1;
	    note.pitch = current_pitch;
	    fwrite(&note, sizeof(note_t), 1, stdout);
	    
    }
    
    return current_pitch;
}


float get_fundamental(complex_fract16* frequencies) {
	return frequency(get_fundamental_idx(frequencies, FFT_SIZE));
}


int peak_comparedesc_val(const void* peak1, const void* peak2) {
    peak_t *p1 = (peak_t*) peak1,  *p2 = (peak_t*) peak2;
    return (p1->val > p2->val) ? -1 : ((p1->val < p2->val) ? 1 : 0);
}
int peak_compareasc_idx(const void* peak1, const void* peak2) {
    peak_t *p1 = (peak_t*) peak1,  *p2 = (peak_t*) peak2;
    return p1->idx - p2->idx;
}
short get_fundamental_idx(complex_fract16* frequencies, int array_size) {
	peak_t peaks[FFT_PARTITIONS];
	
	int i; for(i = 0; i < FFT_PARTITIONS; i++) {
	    short maxidx = 0;
	    fract16 max = 0;
	    
	    int j; for(j = 0; j < FFT_PARTITION_SIZE; j++) {
	        short idx = i*FFT_PARTITION_SIZE + j;
	        fract16 cur = cabs_fr16(frequencies[idx]);
	        if(cur > max) {
	            max = cur;
	            maxidx = idx;
	        }
	    }
	    
	    peaks[i].idx = maxidx;
	    peaks[i].val = max;
	}
	
	// sorts the peaks according to the values, indices remain intact, then
	// we take some of the strongest peaks and choose the leftmost as fundamental
	qsort(peaks, FFT_PARTITIONS, sizeof(peak_t), peak_comparedesc_val);
	qsort(peaks, RELEVANT_PEAKS, sizeof(peak_t), peak_compareasc_idx);
	
    if(peaks[0].val <= SILENCE_THRESHOLD)
        return SILENCE;
	
	return peaks[0].idx;
}


bool is_event(pitch_t pitch) {
    // couldn't be simpler, it's just a first attempt
    return previous_pitch != pitch;
}


float frequency(int index) {
    return (index == SILENCE) ? SILENCE : (index * ((float) SAMPLE_RATE)/FFT_SIZE);
}


pitch_t pitch_from_frequency(float frequency) {
    if(frequency == SILENCE)
        return SILENCE;

    // function from frequency in Hz to the piano key number, ripped from Wolfram Alpha
    return MIDI_PIANO_OFFSET + ((int) round(17.312340490667548 * log(0.038525930703974455 * frequency)));
}

